package ch01.json;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.json.JettisonMappedXmlDriver;

/**
 * Beispielklasse zur Demonstration der JSON-Verarbeitung mit XStream
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden 
 */
public class XStreamXmlAndJsonExample
{
    public static void main(final String[] args)
    {
        final List<Person> persons = createDemoPersons();

        final XStream xStreamForXml = createXStreamForXml();
        final XStream xStreamForJson = createXStreamForJson();

        final String personsAsXml = xStreamForXml.toXML(persons);
        final String personsAsJson = xStreamForJson.toXML(persons);
        System.out.println("XML: \n" + personsAsXml);
        System.out.println("\nJSON:\n" + personsAsJson);

        final List<Person> readPersonsFromXml = (List<Person>) xStreamForXml.fromXML(personsAsXml);
        final List<Person> readPersonsFromJson = (List<Person>) xStreamForJson.fromXML(personsAsJson);
        System.out.println("\nRead from XML :\n" + readPersonsFromXml);
        System.out.println("\nRead from JSON:\n" + readPersonsFromJson);
    }

    private static XStream createXStreamForXml()
    {
        final XStream xStream = new XStream();
        xStream.alias("Person", Person.class);
        return xStream;
    }

    private static XStream createXStreamForJson()
    {
        final XStream xStreamJson = new XStream(new JettisonMappedXmlDriver());
        xStreamJson.alias("Person", Person.class);
        return xStreamJson;
    }

    private static List<Person> createDemoPersons()
    {
        final List<String> michasHobbies = Arrays.asList("Hifi", "Java", "Fotografie");
        final List<String> timsHobbies = Arrays.asList("Hifi", "Java");

        final Person micha = new Person("Michael", "Inden", 44, michasHobbies);
        final Person tim = new Person("Tim", "Bötz", 44, timsHobbies);

        return new ArrayList<>(Arrays.asList(micha, tim));
    }
}
