package ch01.xml.jaxb;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;

/**
 * Beispiel zum Marschalling Unmarshalling mit JAXB
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden 
 */
public class JaxbMarshalAndUnMarshalExample
{
    public static void main(final String[] args) throws Exception
    {
        final Person person = new Person("Michael", "Inden", 44);
        final String filename = "resources/ch01/xml/jaxb/person.xml";
        writePerson(person, filename);

        final Person readPerson = readPerson(filename);
        System.out.println("read Person: " + readPerson);
    }

    public static void writePerson(final Person person, final String filename) throws Exception
    {
        final JAXBContext context = JAXBContext.newInstance(Person.class);
        final Marshaller marshaller = context.createMarshaller();
        marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);

        try (final OutputStream os = new BufferedOutputStream(new FileOutputStream(filename)))
        {
            marshaller.marshal(person, os);
        }
    }

    public static Person readPerson(final String filename) throws Exception
    {
        final JAXBContext context = JAXBContext.newInstance(Person.class);
        final Unmarshaller unmarshaller = context.createUnmarshaller();

        try (final InputStream is = new BufferedInputStream(new FileInputStream(filename)))
        {
            return (Person) unmarshaller.unmarshal(is);
        }
    }
}
