package ch01.xml.jaxb;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Arrays;
import java.util.List;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;

/**
 * Beispiel zum Marschalling Unmarshalling mit JAXB für Listen
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden 
 */
public class JaxbMarshalAndUnMarshalListExample
{
    public static void main(final String[] args) throws Exception
    {
        final Person person1 = new Person("Michael", "Inden", 44);
        final Person person2 = new Person("Joshua", "Bloch", 54);
        final String filename = "resources/ch01/xml/jaxb/persons.xml";
        writePersons(Arrays.asList(person1, person2), filename);
    
        final List<Person> readPersons = readPersons(filename);
        System.out.println("read Persons: " + readPersons);
    }

    private static void writePersons(final List<Person> persons, final String filename) throws Exception
    {
        final JAXBContext context = JAXBContext.newInstance(Personen.class);
        final Marshaller marshaller = context.createMarshaller();
        marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);

        final Personen personen = new Personen();
        personen.setPersons(persons);

        try (final OutputStream os = new BufferedOutputStream(new FileOutputStream(filename)))
        {
            marshaller.marshal(personen, os);
        }
    }

    private static List<Person> readPersons(final String filename) throws Exception
    {
        final JAXBContext context = JAXBContext.newInstance(Personen.class);
        final Unmarshaller unmarshaller = context.createUnmarshaller();

        try (final InputStream is = new BufferedInputStream(new FileInputStream(filename)))
        {
            final Personen personen = (Personen) unmarshaller.unmarshal(is);
            return personen.getPersons();
        }
    }
}
