package ch01.xml.jaxb.stax;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * Beispiel zur Objektrekonstruktion beim Lesen mit DOM 
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden 
 */
public class DomReadPersonExample
{
    public static void main(final String[] args) throws SAXException, IOException, ParserConfigurationException
    {
        final String input = "resources/ch01/xml/jaxb/stax/Personen.xml";

        final DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        final DocumentBuilder builder = factory.newDocumentBuilder();
        final Document doc = builder.parse(input);

        final List<Person> persons = extractPersonsFrom(doc);
        System.out.println("Extracted persons: " + persons);
    }

    private static List<Person> extractPersonsFrom(final Document doc)
    {
        final List<Person> persons = new ArrayList<>();

        final NodeList personElements = doc.getElementsByTagName("Person");
        for (int i = 0; i < personElements.getLength(); i++)
        {
            final Element personElement = (Element) personElements.item(i);
            final NamedNodeMap attributes = personElement.getAttributes();

            persons.add(extractPerson(attributes));
        }

        return persons;
    }

    private static Person extractPerson(final NamedNodeMap attributes)
    {
        return new Person(getValue(attributes, "vorname"), getValue(attributes, "name"),
                          Integer.valueOf(getValue(attributes, "alter")));
    }

    private static String getValue(final NamedNodeMap attributes, final String name)
    {
        return attributes.getNamedItem(name).getNodeValue();
    }
}
