package ch01.xml.jaxb.stax;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLEventWriter;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;

import ch01.xml.jaxb.stax.Person;

/**
 * Beispiel zum Verändern eines DOkuments während des Einlesens 
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden 
 */
public class OnTheFlyModificationExample
{
    public static void main(final String[] args)
                    throws FileNotFoundException, IOException, XMLStreamException, JAXBException
    {
        final String PATH = "resources/ch01/xml/jaxb/stax/";
      
        try (final InputStream in = new FileInputStream(PATH + "Personen.xml");
             final OutputStream out = new FileOutputStream(PATH + "Modified.xml"))
        {
            final XMLInputFactory xif = XMLInputFactory.newInstance();
            final XMLOutputFactory xof = XMLOutputFactory.newInstance();

            final XMLEventReader xer = xif.createXMLEventReader(in);
            final XMLEventWriter xew = xof.createXMLEventWriter(out);

            try
            {
                final JAXBContext jc = JAXBContext.newInstance(Person.class);

                final Unmarshaller unmarshaller = jc.createUnmarshaller();
                final Marshaller marshaller = jc.createMarshaller();
                marshaller.setProperty(Marshaller.JAXB_FRAGMENT, true);

                while (xer.hasNext())
                {
                    processElement(xer, xew, unmarshaller, marshaller);
                }
            }
            finally
            {
                xer.close();
                xew.close();
            }
        }
    }

    private static void processElement(final XMLEventReader xer, final XMLEventWriter xew,
                                       final Unmarshaller unmarshaller, final Marshaller marshaller)
                    throws XMLStreamException, JAXBException
    {
        if (isPersonElement(xer))
        {
            final Person currentPerson = (Person) unmarshaller.unmarshal(xer);
            System.out.println("Read: " + currentPerson);

            if (!isMichael(currentPerson))
            {
                currentPerson.setAlter(currentPerson.getAlter() - 10);

                marshaller.marshal(currentPerson, xew);
                System.out.println("Wrote: " + currentPerson);
            }
        }
        else
        {
            // Durchleiten nicht relevanter ELemente
            xew.add(xer.nextEvent());
        }
    }

    private static boolean isPersonElement(final XMLEventReader xer) throws XMLStreamException
    {
        return xer.peek().isStartElement() && xer.peek().asStartElement().getName().getLocalPart().equals("Person");
    }

    private static boolean isMichael(final Person currentPerson)
    {
        return currentPerson.getVorname().equals("Michael");
    }
}
