package ch01.xml.jaxb.stax;

import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.stream.FactoryConfigurationError;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;
import javax.xml.transform.stream.StreamSource;


/**
 * Beispiel zum Lesen von XML mit der Kombination von StAX und JAXB
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden 
 */
public class PersonUnmarshalWithStAXandJAXBExample
{
    public static void main(final String[] args) throws Exception
    {
        final String input = "resources/ch01/xml/jaxb/stax/Personen.xml";
        final List<Person> persons = readPersons(input);

        System.out.println(persons);
    }

    protected static List<Person> readPersons(final String input)
                    throws FactoryConfigurationError, XMLStreamException, JAXBException
    {
        final StreamSource xml = new StreamSource(input);
        final XMLInputFactory xif = XMLInputFactory.newInstance();
        final XMLStreamReader xsr = xif.createXMLStreamReader(xml);

        try
        {
            final List<Person> persons = new ArrayList<>();
            while (xsr.hasNext())
            {
                xsr.next();

                if (isPersonElement(xsr))
                {
                    final Person person = extractPersonWithJAXB(xsr);
                    persons.add(person);
                }
            }
            return persons;
        }
        finally
        {
            xsr.close();
        }
    }

    protected static boolean isPersonElement(final XMLStreamReader xsr)
    {
        return xsr.isStartElement() && xsr.getLocalName().equals("Person");
    }

    protected static Person extractPersonWithJAXB(final XMLStreamReader xsr) throws JAXBException
    {
        final JAXBContext jaxbContext = JAXBContext.newInstance(Person.class);
        final Unmarshaller unmarshaller = jaxbContext.createUnmarshaller();

        return unmarshaller.unmarshal(xsr, Person.class).getValue();
    }
}