package ch01.xml.jaxb.stax;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * Beispiel zur Objektrekonstruktion beim Lesen mit SAX 
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden 
 */
public class SaxReadPersonExample
{
    public static void main(final String[] args)
                    throws FileNotFoundException, SAXException, IOException, ParserConfigurationException
    {
        final String input = "resources/ch01/xml/jaxb/stax/Personen.xml";

        final SAXParserFactory factory = SAXParserFactory.newInstance();
        final SAXParser saxParser = factory.newSAXParser();

        final PersonExtractHandler handler = new PersonExtractHandler();
        saxParser.parse(new File(input), handler);

        System.out.println("Extracted persons: " + handler.persons);
    }

    private static class PersonExtractHandler extends DefaultHandler
    {
        final List<Person> persons = new ArrayList<>();

        @Override
        public void startElement(final String uri, final String localName, final String qName,
                                 final Attributes attributes)
                        throws SAXException
        {
            if (qName.equalsIgnoreCase("Person"))
            {
                persons.add(extractPerson(attributes));
            }
        }

        private Person extractPerson(final Attributes attributes)
        {
            return new Person(attributes.getValue("vorname"), attributes.getValue("name"),
                              Integer.valueOf(attributes.getValue("alter")));
        }
    }
}
