package ch01.xml.jaxb.stax;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.PropertyException;
import javax.xml.stream.FactoryConfigurationError;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;


/**
 * Beispiel zum Schreiben von XML mit der Kombination von StAX und JAXB
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden 
 */
public class SoapMarshalExample
{
    public static void main(final String[] args)
                    throws FactoryConfigurationError, XMLStreamException, JAXBException, PropertyException
    {
        final XMLOutputFactory xof = XMLOutputFactory.newInstance();
        final XMLStreamWriter streamWriter = xof.createXMLStreamWriter(System.out);

        try
        {
            streamWriter.writeStartDocument();
            final String namespaceURI = "http://schemas.xmlsoap.org/soap/envelope/";
            streamWriter.writeStartElement("s", "Envelope", namespaceURI);
            streamWriter.writeStartElement("s", "Body", namespaceURI);
            streamWriter.writeStartElement("jp", "Content", "http://www.java-profi.de/soap");

            writePersonWithJAXB(streamWriter, new Person("Michael", "Inden", 44));
            streamWriter.writeEndDocument();
        }
        finally
        {
            streamWriter.close();
        }
    }

    private static void writePersonWithJAXB(final XMLStreamWriter streamWriter, final Person person)
                    throws JAXBException, PropertyException
    {
        final JAXBContext jaxbContext = JAXBContext.newInstance(Person.class);
        final Marshaller marshaller = jaxbContext.createMarshaller();
        marshaller.setProperty(Marshaller.JAXB_FRAGMENT, true);

        marshaller.marshal(person, streamWriter);
    }
}