package ch01.xml.jaxp.dom;

import java.io.File;
import java.io.IOException;
import java.util.StringJoiner;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * Beispiel zum Parsen mit DOM
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden 
 */
public class FirstDomExample
{
    public static void main(final String[] args)
    {
        final File xmlFile = new File("resources/ch01/xml/jaxp/dom/Personen.xml");

        final DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

        try
        {
            final DocumentBuilder docBuilder = factory.newDocumentBuilder();
            final Document doc = docBuilder.parse(xmlFile);

            processDocument(doc);
        }
        catch (final ParserConfigurationException | SAXException | IOException e)
        {
            System.out.println("Error processing xml: " + e);
        }
    }

    private static void processDocument(final Document doc)
    {
        final Element root = doc.getDocumentElement();
        final NodeList nodes = root.getChildNodes();

        for (int i = 0; i < nodes.getLength(); i++)
        {
            printNodeInfos(nodes.item(i));
        }
    }

    public static void printNodeInfos(final Node node)
    {
        // Überspringe leere Text-Nodes
        if (node.getNodeType() == Node.TEXT_NODE && node.getNodeValue().trim().length() == 0)
        {
            return;
        }

        System.out.print("Node-Name:  " + node.getNodeName() + ", ");
        System.out.print("Node-Value: " + node.getNodeValue() + ", ");
        System.out.println("Node-Type: " + node.getNodeType());

        final NamedNodeMap attributes = node.getAttributes();
        // API-Unschönheit: liefert null statt leerer Map
        if (attributes != null)
        {
            System.out.print("Attributes: ");
            // JDK 8: StringJoiner
            final StringJoiner joiner = new StringJoiner(", ");
            for (int i = 0; i < attributes.getLength(); i++)
            {
                joiner.add(attributes.item(i).toString());
            }
            System.out.println(joiner.toString());
        }
    }
}
