package ch01.xml.jaxp.dom;

import java.io.File;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Comment;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;

/**
 * Beispiel zum Modifikation eines XML-DOkuments mit DOM
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden 
 */
public class ParseAndModifyDOMExample
{
    private static final Logger logger = Logger.getLogger("ParseAndModifyDOMExample");

    public static void main(final String[] args)
    {
        final String SRC_FILE_NAME = "resources/ch01/xml/jaxp/dom/Personen.xml";
        final String DST_FILE_NAME = "resources/ch01/xml/jaxp/dom/PersonenModified.xml";

        final DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        try
        {
            final DocumentBuilder docBuilder = factory.newDocumentBuilder();
            final Document doc = docBuilder.parse(new File(SRC_FILE_NAME));

            final Element person = createPersonElement(doc, "Mike", "Inden", 44);
            final Comment commentNode = doc.createComment("Dieser Kommentar wird " + "neu eingefügt!");
            attachInfosToDocument(doc, person, commentNode);
            writeDomToFile(DST_FILE_NAME, doc);
        }
        catch (final ParserConfigurationException | SAXException e)
        {
            logger.log(Level.WARNING, "Error parsing file: " + SRC_FILE_NAME, e);
        }
        catch (final TransformerException e)
        {
            throw new IllegalStateException("transformer not working properly", e);
        }
        catch (final IOException e)
        {
            logger.log(Level.SEVERE, "Error reading or writing file", e);
        }
    }

    private static void attachInfosToDocument(final Document doc, final Element personElement,
                                              final Comment commentNode)
    {
        final Element root = doc.getDocumentElement();
        root.appendChild(personElement);
        root.appendChild(commentNode);
    }

    private static Element createPersonElement(final Document doc, final String vorname, final String name,
                                               final int age)
    {
        final Element personElement = doc.createElement("Person");

        personElement.setAttribute("vorname", vorname);
        personElement.setAttribute("name", name);
        personElement.setAttribute("alter", "" + age);

        return personElement;
    }

    private static void writeDomToFile(final String fileName, final Document doc)
                    throws TransformerFactoryConfigurationError, TransformerConfigurationException, TransformerException
    {
        final File xmlOut = new File(fileName);
        final StreamResult outputTarget = new StreamResult(xmlOut);
        final DOMSource xmlSource = new DOMSource(doc);

        final TransformerFactory tranformerFactory = TransformerFactory.newInstance();
        final Transformer transformer = tranformerFactory.newTransformer();
        transformer.transform(xmlSource, outputTarget);
    }
}
