package ch01.xml.jaxp.sax;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * Konverter zur einfachen Aufbereitung von XML als HTML mit SAX
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden 
 */
public class SimpleXmlToHtmlConverter extends DefaultHandler 
{
	private final StringBuilder sb = new StringBuilder();
	private boolean useAsHeader = true;
	
	@Override
	public void startDocument() throws SAXException 
	{
		sb.append("<HTML>" + "\n");
		sb.append("    <BODY>" + "\n");
		sb.append("        <TABLE BORDER='1'>" + "\n");
	}

	@Override
	public void endDocument() throws SAXException 
	{
		sb.append("        </TABLE>" + "\n");
		sb.append("    </BODY>" + "\n");
		sb.append("</HTML>" + "\n");
	}

	@Override
	public void startElement(final String uri, final String localName, final String qName,
			Attributes attributes) throws SAXException {

		if (qName.equals("Person")) 
		{
			handleAttributesAsHeaderOnce(attributes);

			sb.append("<TR>" + "\n");
			handleAttributes(attributes);
		}
	}

	@Override
	public void endElement(final String uri, final String localName, final String qName)
			throws SAXException {

		if (qName.equals("Person")) 
		{
			sb.append("</TR>" + "\n");
		}
	}

	public String contentAsHtml() 
	{
		return sb.toString();
	}
	
	private void handleAttributesAsHeaderOnce(final Attributes atts) 
	{
		if (useAsHeader)
		{
			for (int i = 0; i < atts.getLength(); i++) 
			{
				sb.append("<TH>" + atts.getLocalName(i) + "</TH>" + "\n");
			}
			useAsHeader = false;
		}
	}

	private void handleAttributes(final Attributes atts) 
	{
		for (int i = 0; i < atts.getLength(); i++) 
		{
			sb.append("<TD>" + atts.getValue(i) + "</TD>" + "\n");
		}
	}
}
