package ch01.xml.jaxp.stax;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.StringJoiner;

import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;

/**
 * Beispiel zum Lesen und Parsen eines XML mit StAX
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden 
 */
public class StAXCursorBasedReaderExample
{
    public static void parseXmlWithStAX(final XMLStreamReader xmlStreamReader) throws XMLStreamException
    {
        while (xmlStreamReader.hasNext())
        {
            final int event = xmlStreamReader.next();
            switch (event)
            {
                case XMLStreamConstants.START_ELEMENT:
                    System.out.println("Start: " + xmlStreamReader.getLocalName());
                    System.out.println("Attributes: " + readAttributes(xmlStreamReader));
                    break;
                case XMLStreamConstants.END_ELEMENT:
                    System.out.println("End: " + xmlStreamReader.getLocalName());
                    break;
            }
        }
    }

    protected static String readAttributes(final XMLStreamReader xmlStreamReader)
    {
        final int attributeCount = xmlStreamReader.getAttributeCount();

        final StringJoiner joiner = new StringJoiner(", ");
        for (int i = 0; i < attributeCount; i++)
        {
            joiner.add(xmlStreamReader.getAttributeLocalName(i) + "=" + xmlStreamReader.getAttributeValue(i));
        }
        return joiner.toString();
    }

    public static void main(final String[] args) throws XMLStreamException, IOException
    {
        final File file = new File("resources/ch01/xml/jaxp/stax/Personen.xml");
        try (final FileReader reader = new FileReader(file))
        {
            final XMLInputFactory factory = XMLInputFactory.newInstance();
            final XMLStreamReader xmlStreamReader = factory.createXMLStreamReader(reader);

            parseXmlWithStAX(xmlStreamReader);      
            xmlStreamReader.close();
        }
    }
}