package ch01.xml.jaxp.stax;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;

import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.events.StartElement;
import javax.xml.stream.events.XMLEvent;

/**
 * Beispiel zum Lesen und Parsen eines XML mit StAX
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden 
 */
public class StAXEventReaderExample
{
    public static void parseXmlWithStAX(final XMLEventReader xmlEventReader) throws XMLStreamException
    {
        while (xmlEventReader.hasNext())
        {
            final XMLEvent xmlEvent = xmlEventReader.nextEvent();

            if (xmlEvent.isStartElement())
            {
                final StartElement startElement = xmlEvent.asStartElement();
                System.out.println("Start: " + startElement);
            }
            if (xmlEvent.isEndElement())
            {
                System.out.println("End: " + xmlEvent.asEndElement());
            }
        }
    }

    public static void main(final String[] args) throws XMLStreamException, IOException
    {
        final File file = new File("resources/ch01/xml/jaxp/stax/Personen.xml");
        try (final FileReader reader = new FileReader(file))
        {
    
            final XMLInputFactory factory = XMLInputFactory.newInstance();
            final XMLEventReader xmlStreamReader = 
                                 factory.createXMLEventReader(reader);
    
            parseXmlWithStAX(xmlStreamReader);
            xmlStreamReader.close();
        }
    }
}