package ch01.xml.serialize;

import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamWriter;

/**
 * Beispiel zur Serialisierrung von XML mit XMLStreamWriter
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden 
 */
public class PersonsXmlStreamWriterExample
{
    public static void main(final String[] args) throws Exception
    {
        final Person mike = new Person("Mike", 44);
        final Person lili = new Person("Lili", 36);

        final List<Person> persons = new ArrayList<>(Arrays.asList(mike, lili));
        write(persons, "resources/ch01/xml/serialize/personsByStreamWriter.xml");
    }

    public static void write(final List<Person> persons, final String filename) throws Exception
    {
        try (final OutputStream os = new BufferedOutputStream(new FileOutputStream(filename)))
        {
            final XMLOutputFactory outFactory = XMLOutputFactory.newInstance();
            final XMLStreamWriter writer = outFactory.createXMLStreamWriter(os);

            try
            {
                writer.writeStartDocument();
                writer.writeCharacters("\n");
                writer.writeStartElement("persons");
                writer.writeCharacters("\n");

                for (final Person person : persons)
                {
                    writer.writeStartElement("person");
                    writer.writeAttribute("name", person.getName());
                    writer.writeAttribute("age", String.valueOf(person.getAge()));
                    writer.writeEndElement();
                    writer.writeCharacters("\n");
                }

                writer.writeEndElement();
                writer.writeEndDocument();
            }
            finally
            {
                writer.close();
            }
        }
    }
}
