package ch01.xml.serialize;

import java.beans.XMLDecoder;
import java.beans.XMLEncoder;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Beispiel zur (De-)Serialisierrung von XML mit XMLDeconder/XMLEncoder
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden 
 */
public class XMLEncoderDecoderExample
{
    public static void main(final String[] args) throws Exception
    {
        final Person mike = new Person("Mike", 44);
        final Person lili = new Person("Lili", 36);
        final List<Person> persons = new ArrayList<>(Arrays.asList(mike, lili));

        final String filename = "resources/ch01/xml/serialize/persons.xml";
        write(persons, filename);

        final List<Person> readPersons = read(filename);
        System.out.println("read Persons: " + readPersons);
    }

    public static void write(final List<Person> persons, final String filename) throws Exception
    {
        try (final OutputStream os = new BufferedOutputStream(new FileOutputStream(filename));
                        final XMLEncoder encoder = new XMLEncoder(os))
        {
            encoder.writeObject(persons);
        }
    }

    public static List<Person> read(final String filename) throws Exception
    {
        try (final InputStream is = new BufferedInputStream(new FileInputStream(filename));
                        final XMLDecoder decoder = new XMLDecoder(is))
        {
            return (List<Person>) decoder.readObject();
        }
    }
}