package ch01.xml.transform;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Map;
import java.util.TreeMap;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.w3c.dom.Document;
import org.xml.sax.SAXException;

/**
 * Beispiel zur Rück-Transformation von XML per XSLT
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden 
 */
public class XmlBackTransformationExample
{
    private static final Map<String, String> transformations = new TreeMap<>();

    static
    {
        transformations.put("KundenstammToPersonen.xsl", "Personen.xml");
    }

    public static void main(final String[] args)
    {
        final String BASE_PATH = "resources/ch01/xml/transform/";
        final File xmlFile = new File(BASE_PATH + "output/Kundenstamm.xml");

        try
        {
            final DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            final DocumentBuilder docBuilder = factory.newDocumentBuilder();
            final Document doc = docBuilder.parse(xmlFile);

            transformations.forEach((xslfile, destfile) -> performTransformation(doc, BASE_PATH + xslfile,
                                                                                 BASE_PATH + "output/" + destfile));
        }
        catch (final ParserConfigurationException | SAXException | IOException e)
        {
            System.out.println("Errors processing file: " + xmlFile.getAbsolutePath());
        }
    }

    private static void performTransformation(final Document doc, final String xslfileName, final String destfileName)
    {
        final File xmlOut = new File(destfileName);
        final StreamResult outputTarget = new StreamResult(xmlOut);
        final DOMSource xmlSource = new DOMSource(doc);

        try (final InputStream xslInputStream = new FileInputStream(xslfileName))
        {
            final Source xslSource = new StreamSource(xslInputStream);
            final TransformerFactory tranformerFactory = TransformerFactory.newInstance();
            final Transformer transformer = tranformerFactory.newTransformer(xslSource);

            transformer.transform(xmlSource, outputTarget);
        }
        catch (final IOException | TransformerFactoryConfigurationError | TransformerException e)
        {
            System.out.println("Errors processing xsl: " + xslfileName);
        }
    }
}
