package ch01.xml.transform;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.w3c.dom.Document;
import org.xml.sax.SAXException;

/**
 * Beispiel zur Transformation von XML per XSLT
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden 
 */
public class XmlTransformationsExample
{
    public static void main(final String[] args)
    {
        final String PATH = "resources/ch01/xml/transform/";
        final File xmlFile = new File(PATH + "Personen.xml");

        try
        {
            final DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            final DocumentBuilder docBuilder = factory.newDocumentBuilder();
            final Document doc = docBuilder.parse(xmlFile);

            transformXmlDoc(doc, PATH + "PersonenToCsv.xsl", PATH + "output/" + "Personen.csv");
            transformXmlDoc(doc, PATH + "PersonenToHtml.xsl", PATH + "output/" + "Personen.html");
            transformXmlDoc(doc, PATH + "PersonenToKundenstamm.xsl", PATH + "output/" + "Kundenstamm.xml");
        }
        catch (final ParserConfigurationException | SAXException | IOException e)
        {
            System.out.println("Error processing file: " + e);
        }
    }

    private static void transformXmlDoc(final Document doc, final String xslFileName, final String destFileName)
    {
        final File xmlOut = new File(destFileName);
        final StreamResult outputTarget = new StreamResult(xmlOut);
        final DOMSource xmlSource = new DOMSource(doc);

        try (final InputStream xslInputStream = new FileInputStream(xslFileName))
        {
            final Source xslSource = new StreamSource(xslInputStream);

            final TransformerFactory transformerFactory = TransformerFactory.newInstance();
            final Transformer transformer = transformerFactory.newTransformer(xslSource);

            transformer.transform(xmlSource, outputTarget);
        }
        catch (final IOException | TransformerFactoryConfigurationError | TransformerException e)
        {
            System.out.println("Error processing xsl: " + e);
        }
    }
}
