package ch01.xml.xpath;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import ch01.xml.jaxp.dom.FirstDomExample;

/**
 * Beispielprogramm zu Auswertungen mit XPath   
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden 
 */
public class XPathQueryExample
{
    public static void main(final String[] args)
                    throws XPathExpressionException, ParserConfigurationException, SAXException, IOException
    {
        // DOM
        final DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        final DocumentBuilder builder = factory.newDocumentBuilder();
        final Document doc = builder.parse(XPathQueryExample.class.getResourceAsStream("Personen.xml"));

        // XPath
        final XPathFactory xFactory = XPathFactory.newInstance();
        final XPath xpath = xFactory.newXPath();

        // Queries
        System.out.println("performXPathQueries");
        System.out.println("-------------------");
        performXPathQueries(doc, xpath);

        final long count = countMichaels(doc, xpath);
        System.out.println("\ncountMichaels => " + count);

        final int age = 40;
        final boolean check = twoOrMorePersonsOlderThan(doc, xpath, age);
        System.out.println("\n2 or more older than " + age + " years? " + check);
    }

    private static void performXPathQueries(final Document doc, final XPath xpath) throws XPathExpressionException
    {
        final Map<String, QName> queriesAndTypes = new LinkedHashMap<>();
        queriesAndTypes.put("/Personen/Person", XPathConstants.NODESET);
        queriesAndTypes.put("/Personen/Person[2]", XPathConstants.NODE);
        queriesAndTypes.put("/*/Person", XPathConstants.NODESET);
        queriesAndTypes.put("/*/Person/@name", XPathConstants.NODESET);

        for (final Map.Entry<String, QName> entry : queriesAndTypes.entrySet())
        {
            final List<Node> nodes = performQuery(entry.getKey(), doc, xpath, entry.getValue());
            nodes.forEach(FirstDomExample::printNodeInfos);
        }
    }

    private static List<Node> performQuery(final String query, final Document doc, final XPath xpath, final QName mode)
                    throws XPathExpressionException
    {
        System.out.println("\nQuery: " + query);

        final XPathExpression expr = xpath.compile(query);

        if (mode == XPathConstants.NODESET)
        {
            final List<Node> nodes = new ArrayList<>();

            final NodeList nodeList = (NodeList) expr.evaluate(doc, mode);
            for (int i = 0; i < nodeList.getLength(); i++)
            {
                final Node node = nodeList.item(i);
                nodes.add(node);
            }

            return nodes;
        }
        if (mode == XPathConstants.NODE)
        {
            final Node node = (Node) expr.evaluate(doc, mode);
            return Collections.singletonList(node);
        }

        return Collections.emptyList();
    }

    private static long countMichaels(final Document doc, final XPath xpath) throws XPathExpressionException
    {
        final String countMikes = "count(//Person[@vorname='Michael'])";
        final XPathExpression expr = xpath.compile(countMikes);

        return ((Double) expr.evaluate(doc, XPathConstants.NUMBER)).longValue();
    }

    private static boolean twoOrMorePersonsOlderThan(final Document doc, final XPath xpath, final int age)
                    throws XPathExpressionException
    {
        final String moreThan2Persons = "count(//Person[@alter >= " + age + "]) >= 2";
        final XPathExpression expr = xpath.compile(moreThan2Persons);

        return (Boolean) expr.evaluate(doc, XPathConstants.BOOLEAN);
    }
}