package ch03.jdbc;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.apache.log4j.Logger;

/**
 * Utility-Klasse zum einfacheren Zugriff auf Datenbankfunktionalitaeten
 * 
 * @author Michael Inden
 * 
 * Copyright 2012, 2016 by Michael Inden 
 */
public final class DbUtils
{
    private static final Logger logger = Logger.getLogger(DbUtils.class);

    public static void logSqlExceptionAndWarningInfos(final SQLException ex)
    {
        logger.error(ex.getClass().getSimpleName() + " occured\n", ex);
        logSqlExceptionAndWarningInfosImpl(ex);

        SQLException followingException = ex.getNextException();
        while (followingException != null)
        {
            logSqlExceptionAndWarningInfosImpl(followingException);

            followingException = followingException.getNextException();
        }
    }

    private static void logSqlExceptionAndWarningInfosImpl(final SQLException ex)
    {
        logger.error("Message:   " + ex.getLocalizedMessage());
        logger.error("SQLState:   " + ex.getSQLState());
        logger.error("ErrorCode:   " + ex.getErrorCode());
    }

    public static void printSqlExceptionAndWarningInfos(final SQLException ex)
    {
        System.out.println(ex.getClass().getSimpleName() + " occured\n");
        printSqlExceptionAndWarningInfosImpl(ex);

        SQLException followingException = ex.getNextException();
        while (followingException != null)
        {
            printSqlExceptionAndWarningInfosImpl(followingException);

            followingException = followingException.getNextException();
        }
    }

    private static void printSqlExceptionAndWarningInfosImpl(final SQLException ex)
    {
        System.out.println("Message:   " + ex.getLocalizedMessage());
        System.out.println("SQLState:   " + ex.getSQLState());
        System.out.println("ErrorCode:   " + ex.getErrorCode());
    }

    public static void safeCloseConnection(final Connection con)
    {
        if (con != null)
        {
            try
            {
                con.close();
            }
            catch (final SQLException e)
            {
                logSqlExceptionAndWarningInfos(e);
            }
        }
    }

    public static void safeCloseStatement(final Statement stmt)
    {
        if (stmt != null)
        {
            try
            {
                stmt.close();
            }
            catch (final SQLException e)
            {
                logSqlExceptionAndWarningInfos(e);
            }
        }
    }

    public static void safeCloseResultSet(final ResultSet rs)
    {
        if (rs != null)
        {
            try
            {
                rs.close();
            }
            catch (final SQLException e)
            {
                logSqlExceptionAndWarningInfos(e);
            }
        }
    }
}
