package ch03.jdbc;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Set;
import java.util.TreeSet;

/**
 * Beispielklasse zur Demonstration des einfachen Zugriffs per JDBC auf eine Datenbank
 * 
 * @author Michael Inden
 * 
 * Copyright 2012, 2016 by Michael Inden 
 */
public final class FirstJdbcExample
{
    // Achtung: Hier als Vereinfachung nur rudimentäre Fehlerbehandlung 
    public static void main(final String[] args) throws SQLException, 
                                                        ClassNotFoundException
    {
       // Datenbanktreiber laden (optional seit JDBC 4)  
       Class.forName("org.hsqldb.jdbcDriver"); 
    
       // Datenbankverbindungseinstellungen definieren  
       final String dbUrl = "jdbc:hsqldb:hsql://localhost/java-profi";
       final String dbUsername = "sa";
       final String dbPassword = "";
    
       Connection connection = null;
       Statement statement = null;
       ResultSet resultSet = null;
       
       try
       {
           // Datenbankverbindung aufbauen  
           connection = DriverManager.getConnection (dbUrl, 
                                                     dbUsername, 
                                                     dbPassword);
        
           // SQL-Befehlsobjekt erstellen 
           statement = connection.createStatement();
        
           // SQL-Abfrage ausführen 
           resultSet = statement.executeQuery("SELECT * FROM Personen");
        
           // Ergebnismenge auswerten und speichern, JDK 7: Diamond Operator 
           final Set<String> names = new TreeSet<>();
           while (resultSet.next())
           {
               names.add(resultSet.getString("Name"));
           }
                      
           // Ausgabe der ermittelten Nachnamen mit JDK 8 forEach() und Lambda
           names.forEach(name -> System.out.println("Nachname='" + name + "'"));
       }
       finally
       {          
           // Achtung: Nicht ganz korrekte Implementierung von Aufräumarbeiten
           if (resultSet!= null) resultSet.close();
           if (statement!= null) statement.close();
           if (connection!= null) connection.close();
       }
    }
}

