package ch03.jdbc;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.Set;
import java.util.TreeSet;

/**
 * BeispielKlasse zur Demonstration des einfachen Zugriffs per JDBC auf eine Datenbank.
 * Hier jetzt unter Einsatz er Klasse DbProperties.
 * 
 * @author Michael Inden
 * 
 * Copyright 2012 by Michael Inden 
 */
public final class FirstJdbcExampleWithDbProperties
{
    public static void main(String[] args) throws Exception
    {
        // Datenbanktreiber laden (optional seit JDBC 4)  
        Class.forName("org.hsqldb.jdbcDriver"); 
        
        Connection connection = null;
        Statement statement = null;
        ResultSet resultSet = null;
        
        try
        {
            final DbProperties dbProperties = new DbProperties();
            connection = DriverManager.getConnection(dbProperties.getUrl(), dbProperties.getUserName(), dbProperties.getPassword());

            // SQL-Befehlsobjekt erstellen 
            statement = connection.createStatement();

            // SQL-Befehl ausführen 
            resultSet = statement.executeQuery("SELECT * " + "FROM Personen;");

            // Ergebnismenge auswerten und speichern 
            final Set<String> lastNames = new TreeSet<>();
            while (resultSet.next())
                lastNames.add(resultSet.getString("Name"));

            // Ausgabe der ermittelten Nachnamen 
            for (final String lastName : lastNames)
                System.out.println("Nachname='" + lastName + "'");
        }
        finally
        {
            // Alle Ressourcen wieder schlie�en, aber Achtung: siehe Text
            DbUtils.safeCloseResultSet(resultSet);
            DbUtils.safeCloseStatement(statement);
            DbUtils.safeCloseConnection(connection);
        }
    }
}