package ch03.jdbc;

import java.sql.Connection;
import java.sql.Date;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.Map;
import java.util.TreeMap;

/**
 * BeispielKlasse zur Demonstration des einfachen Zugriffs per JDBC auf eine Datenbank
 * und Auslesen der Daten mithilfe eines ResultSets.

 * @author Michael Inden
 * 
 * Copyright 2012, 2016 by Michael Inden 
 */
public final class JdbcResultSetExample
{
    public static void main(final String[] args) throws Exception
    {
        Class.forName("org.hsqldb.jdbcDriver"); 
        
        Connection connection = null;
        Statement statement = null;
        
        try
        {
            final DbProperties dbProperties = new DbProperties();
            connection = DriverManager.getConnection(dbProperties.getUrl(), dbProperties.getUserName(), dbProperties.getPassword());

            statement = connection.createStatement();

			final Map<String, Date> namesAndBirthdays = new TreeMap<>();
			
			final String sqlQueryCommand = "SELECT Vorname, Geburtstag FROM Personen";            
			try (ResultSet resultSet = statement.executeQuery(sqlQueryCommand))
			{            
			    while (resultSet.next())
			    {
			    	namesAndBirthdays.put(resultSet.getString(1), resultSet.getDate("Geburtstag"));
			    }
			}
			    
			// Ausgabe der ermittelten Paare Vorname => Geburtstag 
			namesAndBirthdays.forEach((k,v) -> System.out.println("Vorname='" + k + "' / " +
			                                                      "'Geburtstag: " + v));
        }
        finally
        {
            // Alle Ressourcen wieder schließen, aber Achtung: siehe Text
            DbUtils.safeCloseStatement(statement);
            DbUtils.safeCloseConnection(connection);
        }
    }
}