package ch03.jdbc;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * BeispielKlasse zur Demonstration des einfachen Zugriffs per JDBC auf eine Datenbank
 * und Modifizieren der Daten mithilfe eines ResultSets.
 * 
 * @author Michael Inden
 * 
 * Copyright 2012, 2016 by Michael Inden 
 */
public final class JdbcResultSetUpdateExample
{
    private static void updatePersons(final Connection connection, final String columnName,
                                      final String originalValue, final String newValue) throws SQLException
    {
        final String sqlQuery = "SELECT * FROM Personen";

        // JDK 7: ARM  
        try (final Statement statement = connection.createStatement(
                ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE);
             // Änderungen am ResultSet vornehmen  
             final ResultSet resultSet = statement.executeQuery(sqlQuery))
        {
            while (resultSet.next())
            {
                final String value = resultSet.getString(columnName);
                if (value.equals(originalValue))
                {
                    // Änderungen am ResultSet vornehmen  
                    resultSet.updateString(columnName, newValue);
                    // Änderungen in DB schreiben  
                    resultSet.updateRow();
                }
            }
        }
    }

    private static void prinPersons(final Connection connection) throws SQLException
    {
        final String sqlQuery = "SELECT Vorname, Geburtstag FROM Personen";            

        final List<List<?>> namesAndBirthdays = new ArrayList<>();

        try (final Statement statement = connection.createStatement(
                ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE);
             // Änderungen am ResultSet vornehmen  
             final ResultSet resultSet = statement.executeQuery(sqlQuery))
        {
            while (resultSet.next())
            {
                namesAndBirthdays.add(Arrays.asList(resultSet.getString(1), resultSet.getDate("Geburtstag")));
            }
        }
        
        // Ausgabe der ermittelten Paare Vorname & Geburtstag
        namesAndBirthdays.forEach((v) -> System.out.println("Vorname='" + v.get(0) + "' / " +
                                                            "'Geburtstag: " + v.get(1)));

    }
    
    public static void main(final String[] args) throws SQLException
    {
        final DbProperties dbProps = new DbProperties();

        // JDK 7: ARM  
        try (final Connection connection = DriverManager.getConnection(dbProps.getUrl(),
                dbProps.getUserName(), dbProps.getPassword()))
        {
            System.out.println("Before modifications:");
            prinPersons(connection);
            updatePersons(connection, "Vorname", "Heinz", "Michael");        
            updatePersons(connection, "Vorname", "Werner", "Michael");

            System.out.println("\nAfter modifications:");
            prinPersons(connection);
        }
    }
}
