package ch03.jdbc.orm;

import static ch03.jdbc.orm.ColumnIndices.PersonenColumns.COLUMN_IDX_BIRTHDAY;
import static ch03.jdbc.orm.ColumnIndices.PersonenColumns.COLUMN_IDX_FIRSTNAME;
import static ch03.jdbc.orm.ColumnIndices.PersonenColumns.COLUMN_IDX_NAME;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import ch03.jdbc.DbProperties;

/**
 * Beispiel zum Object Relational Mapping (ORM), in dem Personen aus der Datenbank gelesen werden.
 * 
 * @author Michael Inden
 * 
 * Copyright 2012, 2016 by Michael Inden 
 */
public class GetAllPersonsExample
{
    public static List<Person> getAllPersonsFromDb(final Connection connection) throws SQLException
    {
        final List<Person> persons = new ArrayList<>();

        final String sqlQuery = "SELECT * FROM Personen";
        try (final PreparedStatement preparedStatement = connection.prepareStatement(sqlQuery);
             final ResultSet resultSet = preparedStatement.executeQuery())
        {
            while (resultSet.next())
            {
                final String firstName = resultSet.getString(COLUMN_IDX_FIRSTNAME);
                final String lastName = resultSet.getString(COLUMN_IDX_NAME);
                final Date birthday = resultSet.getDate(COLUMN_IDX_BIRTHDAY);

                persons.add(new Person(firstName, lastName, birthday));
            }
        }

        return persons;
    }

    public static void main(final String[] args) throws SQLException
    {
        final DbProperties dbProps = new DbProperties();

        try (final Connection connection = DriverManager.getConnection(dbProps.getUrl(), dbProps.getUserName(),
                                                                       dbProps.getPassword()))
        {
            final List<Person> persons = getAllPersonsFromDb(connection);
            persons.forEach(System.out::println);
        }
    }
}