package ch03.jdbc.orm;

import static ch03.jdbc.orm.ColumnIndices.AdressenColumns.COLUMN_IDX_CITY;
import static ch03.jdbc.orm.ColumnIndices.AdressenColumns.COLUMN_IDX_NUMBER;
import static ch03.jdbc.orm.ColumnIndices.AdressenColumns.COLUMN_IDX_STREET;
import static ch03.jdbc.orm.ColumnIndices.AdressenColumns.COLUMN_IDX_ZIPCODE;
import static ch03.jdbc.orm.ColumnIndices.PersonenColumns.COLUMN_IDX_BIRTHDAY;
import static ch03.jdbc.orm.ColumnIndices.PersonenColumns.COLUMN_IDX_FIRSTNAME;
import static ch03.jdbc.orm.ColumnIndices.PersonenColumns.COLUMN_IDX_NAME;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import ch03.jdbc.DbProperties;

/**
 * Beispiel zum Object Relational Mapping (ORM), in dem Personen und assozierte Adressen als
 * eine Ergebnissmenge durch Tabellen-JOIN aus der Datenbank gelesen werden.
 * 
 * @author Michael Inden
 * 
 * Copyright 2012, 2016 by Michael Inden 
 */
public class GetAllPersonsWithAddressesJoinExample
{
    public static List<Person> getPersonsWithAddressFromDb(final Connection con) throws SQLException
    {
        final String query = "SELECT * FROM Personen, Adressen " + "WHERE Personen.ID = Adressen.PersonenID;";

        final List<Person> persons = new ArrayList<>();

        try (final PreparedStatement prepStatement = con.prepareStatement(query);
             final ResultSet resultSet = prepStatement.executeQuery())
        {
            while (resultSet.next())
            {
                final String firstName = resultSet.getString(COLUMN_IDX_FIRSTNAME);
                final String lastName = resultSet.getString(COLUMN_IDX_NAME);
                final Date birthday = resultSet.getDate(COLUMN_IDX_BIRTHDAY);

                // Spaltenindizes der Adressen-Tabelle sind um die Spalten der Personen-Tabelle verschoben 
                final int offset = COLUMN_IDX_BIRTHDAY;
                final String street = resultSet.getString(offset + COLUMN_IDX_STREET);
                final String number = resultSet.getString(offset + COLUMN_IDX_NUMBER);
                final int zipcode = resultSet.getInt(offset + COLUMN_IDX_ZIPCODE);
                final String city = resultSet.getString(offset + COLUMN_IDX_CITY);

                final Person person = new Person(firstName, lastName, birthday);
                final Address address = new Address(street, number, zipcode, city);
                person.setAddresses(Collections.singletonList(address));

                persons.add(person);
            }
        }

        return persons;
    }

    public static void main(final String[] args) throws SQLException
    {
        final DbProperties dbProps = new DbProperties();

        try (final Connection connection = DriverManager.getConnection(dbProps.getUrl(), dbProps.getUserName(), dbProps.getPassword()))
        {
            final List<Person> persons = getPersonsWithAddressFromDb(connection);
            persons.forEach(System.out::println);
        }
    }
}