package ch03.jdbc.orm;

import static ch03.jdbc.orm.ColumnIndices.PersonenColumns.COLUMN_IDX_BIRTHDAY;
import static ch03.jdbc.orm.ColumnIndices.PersonenColumns.COLUMN_IDX_FIRSTNAME;
import static ch03.jdbc.orm.ColumnIndices.PersonenColumns.COLUMN_IDX_ID;
import static ch03.jdbc.orm.ColumnIndices.PersonenColumns.COLUMN_IDX_NAME;
import static ch03.jdbc.orm.GetPersonsAddressesExample.getAddressesForPersonenId;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import ch03.jdbc.DbProperties;

/**
 * Beispiel zum Object Relational Mapping (ORM), in dem Personen und assozierte Adressen in einem
 * zweiten Schritt aus der Datenbank gelesen werden.
 * 
 * @author Michael Inden
 * 
 * Copyright 2012, 2016 by Michael Inden 
 */
public final class GetAllPersonsWithAdressesExample
{
    public static List<Person> getAllPersonsWithAddressesFromDb(final Connection connection) throws SQLException
    {
        final String sqlQuery = "SELECT * FROM Personen";

        final List<Person> persons = new ArrayList<>();

        try (final PreparedStatement preparedStatement = connection.prepareStatement(sqlQuery);
             final ResultSet resultSet = preparedStatement.executeQuery())
        {
            while (resultSet.next())
            {
                // ID der Person ermitteln => Fremdschlüssel in Adressen
                final int id = resultSet.getInt(COLUMN_IDX_ID);
                final String firstName = resultSet.getString(COLUMN_IDX_FIRSTNAME);
                final String lastName = resultSet.getString(COLUMN_IDX_NAME);
                final Date birthday = resultSet.getDate(COLUMN_IDX_BIRTHDAY);

                // Person normal erzeugen
                final Person person = new Person(firstName, lastName, birthday);

                // Anschließend Adressen zur Person ermitteln und setzen
                person.setAddresses(getAddressesForPersonenId(connection, id));

                persons.add(person);
            }
        }

        return persons;
    }

    public static void main(final String[] args) throws SQLException
    {
        final DbProperties dbProps = new DbProperties();

        try (final Connection connection = DriverManager.getConnection(dbProps.getUrl(), dbProps.getUserName(), dbProps.getPassword()))
        {
            final List<Person> persons = getAllPersonsWithAddressesFromDb(connection);
            persons.forEach(System.out::println);
        }
    }
}