package ch03.jdbc.orm;

import static ch03.jdbc.orm.ColumnIndices.AdressenColumns.COLUMN_IDX_CITY;
import static ch03.jdbc.orm.ColumnIndices.AdressenColumns.COLUMN_IDX_ZIPCODE;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import ch03.jdbc.DbProperties;

/**
 * Beispiel zum Object Relational Mapping (ORM), in dem Adressen aus der Datenbank gelesen werden,
 * die einer speziellen Person zugeordnet sind.
 * 
 * @author Michael Inden
 * 
 * Copyright 2012, 2016 by Michael Inden 
 */
public final class GetPersonsAddressesExample
{
	public static List<Address> getAddressesForPersonenId(final Connection connection, final int personId) throws SQLException
	{
	    final List<Address> addresses = new ArrayList<>();
	
		final String sqlQuery = "SELECT * FROM Adressen WHERE Adressen.PersonenID = ?";
	
	    try (final PreparedStatement preparedStmt = connection.prepareStatement(sqlQuery))
	    {
	        preparedStmt.setInt(1, personId);
	        try (final ResultSet resultSet = preparedStmt.executeQuery())
	        {
	            while (resultSet.next())
	            {
	                // Zur Demonstration nutze ich hier Magic Numbers statt Konstanten
	                final String street = resultSet.getString(2);
	                final String number = resultSet.getString(3);
	                
	                // Es bietet sich aber an, Konstanten für die Spalten zu verwenden
	                final int zipcode = resultSet.getInt(COLUMN_IDX_ZIPCODE);
	                final String city = resultSet.getString(COLUMN_IDX_CITY);
	
	                addresses.add(new Address(street, number, zipcode, city));
	            }
	        }
	    }        
	
	    return addresses;
	}

    public static void main(final String[] args) throws SQLException
    {
        final DbProperties dbProps = new DbProperties();

        final int personenId = 1;

        try (final Connection connection = DriverManager.getConnection(dbProps.getUrl(), dbProps.getUserName(), dbProps.getPassword()))
        {
            final List<Address> addresses = getAddressesForPersonenId(connection, personenId);
            System.out.println("Got addresses for personId = " + personenId + " from DB: ");
            addresses.forEach(System.out::println);
        }
    }
}