package ch04.jpa.dao;

import java.util.Date;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

/**
 * Beispiel zur Demonstration von Zugriffen auf die Datenbank mithilfe eines DAO und JPA
 * 
 * @author Michael Inden
 * 
 * Copyright 2012, 2106 by Michael Inden 
 */
public final class PersonDaoExample
{
    public static void main(final String[] args) throws Exception
    {
        final EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory("java-profi-PU-DAO");
        final EntityManager entityManager = entityManagerFactory.createEntityManager();

        try
        {
        	performJPAActions(entityManager);
        }
        finally
        {
	        entityManager.close();
	        entityManagerFactory.close();
        }
    }

    private static void performJPAActions(final EntityManager entityManager) 
    {
    	try 
    	{
    		entityManager.getTransaction().begin();
    		
    	    executeStatements(entityManager);
    	    
    		entityManager.getTransaction().commit();
    	}
    	catch (Exception e) 
    	{
    		entityManager.getTransaction().rollback();
    	}
    }
    
private static void executeStatements(final EntityManager entityManager) 
{
	// DAO erzeugen 
    final PersonDAO dao = new PersonDAO(entityManager);

    // Einfügeoperationen ausführen und das Resultat prüfen
    final Person michael = new Person("Micha-DAO", "Inden", new Date(71, 1, 7));
    final Person michael2 = new Person("Micha-DAO", "Inden", new Date(71, 1, 7));
     final Person werner = new Person("Werner-DAO", "Inden", new Date(40, 0, 31));
System.out.println(michael);
     final long michaelId = dao.createPerson(michael);
    final long michaelId2 = dao.createPerson(michael2);
    final long wernerId = dao.createPerson(werner);

    final List<Person> persons2 = dao.findAllPersons();
    persons2.forEach(System.out::println);
    
    // Änderungen ausführen und das Resultat prüfen
    dao.deletePersonById(michaelId);
    werner.setFirstName("Dr. h.c. Werner"); 

    final List<Person> persons = dao.findAllPersons();
    persons.forEach(System.out::println);
}

    private PersonDaoExample()
    {        
    }
}
