package ch04.jpa.dateandtime;

import java.time.LocalDate;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.TypedQuery;

/**
 * Beispiel zur Demonstration der Verarbetung des neuen Date and Time API mit JPA
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden 
 */
public final class PersonDateAndTimeExample
{
    public static void main(final String[] args) throws Exception
    {
        final EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory("java-profi-PU-Date");
        final EntityManager entityManager = entityManagerFactory.createEntityManager();

        try
        {
            performJPAActions(entityManager);
        }
        finally
        {
            entityManager.close();
            entityManagerFactory.close();
        }
    }

    private static void performJPAActions(final EntityManager entityManager)
    {
        try
        {
            entityManager.getTransaction().begin();

            executeStatements(entityManager);

            entityManager.getTransaction().commit();
        }
        catch (final Exception e)
        {
            entityManager.getTransaction().rollback();
        }
    }

    private static void executeStatements(final EntityManager entityManager)
    {
        // Einfügeoperationen ausführen und das Resultat prüfen
        final Person michael = new Person("Micha-Date", "Inden", LocalDate.of(1971, 2, 7));
        final Person werner = new Person("Werner-Date", "Inden", LocalDate.of(1940, 1, 31));

        entityManager.persist(michael);
        entityManager.persist(werner);

        final String jpql = "SELECT person FROM Person person";
        final TypedQuery<Person> typedQuery = entityManager.createQuery(jpql, Person.class);
        typedQuery.getResultList().forEach(System.out::println);
    }
}
