package ch04.jpa.firstexample;

import java.util.Date;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

/**
 * Beispiel zur Demonstration einfacher Zugriffe auf die Datenbank mit JPA
 * 
 * @author Michael Inden
 * 
 * Copyright 2012, 2016 by Michael Inden 
 */
public final class FirstJpaExample
{
    public static void main(final String[] args) throws Exception
    {
        // Zum Applikationsstart: Einlesen der Konfiguration "persistence.xml"
        final EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory("java-profi-PU");

        // Zur Verarbeitung von Daten => EntityManager bietet Zugriff auf Datenbank
        final EntityManager entityManager = entityManagerFactory.createEntityManager();

        try
        {
            // Aktionen auf der DB ausführen 
            performJPAActions(entityManager);
        }
        finally
        {
            // EntityManager wieder freigeben 
            entityManager.close();

            // Am Ende der Applikation 
            entityManagerFactory.close();
        }
    }

    private static void performJPAActions(final EntityManager entityManager)
    {
        try
        {
            entityManager.getTransaction().begin();

            executeStatements(entityManager);

            entityManager.getTransaction().commit();
        }
        catch (final Exception e)
        {
            entityManager.getTransaction().rollback();
        }
    }

    private static void executeStatements(final EntityManager entityManager)
    {
        final Person michael = new Person("Micha-JPA", "Inden", new Date(71, 1, 7));
        entityManager.persist(michael);
    }
}
