package ch04.jpa.firstexample;

import java.util.Date;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

/**
 * Beispiel zur Demonstration einfacher Zugriffe auf die Datenbank mit JPA
 * 
 * @author Michael Inden
 * 
 * Copyright 2012, 2016 by Michael Inden 
 */
public final class JpaCrudExample
{
    public static void main(final String[] args) throws Exception
    {
        // Zum Applikationsstart: Einlesen der Konfiguration "persistence.xml"
        final EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory("java-profi-PU");

        // Zur Verarbeitung von Daten => EntityManager bietet Zugriff auf Datenbank
        final EntityManager entityManager = entityManagerFactory.createEntityManager();
                    
        try
        {
        	// Aktionen auf der DB ausführen 
        	performJPAActions(entityManager);
        }
        finally
        {
        	// EntityManager wieder freigeben 
	        entityManager.close();
	        
	        // Am Ende der Applikation 
	        entityManagerFactory.close();
        }
    }

    private static void performJPAActions(final EntityManager entityManager) 
    {
    	try 
    	{
    		entityManager.getTransaction().begin();
    		
    	    executeStatements(entityManager);
    	    
    		entityManager.getTransaction().commit();
    	}
    	catch (Exception e) 
    	{
    		entityManager.getTransaction().rollback();
    	}
    }

private static void executeStatements(final EntityManager entityManager) 
{
   final Person michael = new Person("Michael", "Inden", new Date(71, 1, 7));
   final Person werner = new Person("Werner", "Inden", new Date(40, 0, 31));
    
   // $\mbox{\bfseries CREATE }$
   entityManager.persist(michael);
   entityManager.persist(werner);
    
   // $\mbox{\bfseries READ }$
   final Long id = werner.getId();
   final Person wernerFromDb = entityManager.find(Person.class, id);

   // $\mbox{\bfseries UPDATE }$
   wernerFromDb.setFirstName("Dr. h.c. Werner"); 

   // $\mbox{\bfseries DELETE }$
   entityManager.remove(michael);
}
}
