package ch04.jpa.jpql_queries;

import java.util.Arrays;
import java.util.Date;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.persistence.TypedQuery;

/**
 * Beispiel zur Demonstration von Objekterzeugungs-Queries mit JPQL
 * 
 * @author Michael Inden
 * 
 * Copyright 2012, 2016 by Michael Inden 
 */
public final class NamedConstructionAndNativeQueriesExample
{
    public static void main(final String[] args) throws Exception
    {
        final EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory("java-profi-PU-JPQL");
        final EntityManager entityManager = entityManagerFactory.createEntityManager();

        try
        {
            performJPAActions(entityManager);
        }
        finally
        {
            entityManager.close();
            entityManagerFactory.close();
        }
    }

    private static void performJPAActions(final EntityManager entityManager)
    {
        try
        {
            entityManager.getTransaction().begin();

            executeStatements(entityManager);

            entityManager.getTransaction().commit();
        }
        catch (final Exception e)
        {
            entityManager.getTransaction().rollback();
        }
    }

    private static void executeStatements(final EntityManager entityManager)
    {
        createPersons(entityManager);

        performNamedQuery(entityManager);
        performObjCreateQuery(entityManager);
        performNativeQuery(entityManager);
    }

    private static void performNamedQuery(final EntityManager entityManager)
    {
        final Query namedQuery = entityManager.createNamedQuery("findPersonsBornAfter");
        namedQuery.setParameter("birthday", new Date(77, 1, 1));

        System.out.println("Named Query");
        final List<Person> result = namedQuery.getResultList();
        result.forEach(System.out::println);
    }

    private static void performObjCreateQuery(final EntityManager entityManager)
    {
        final String createObjJPQL = "SELECT NEW ch04.jpa.jpql_queries.NameAndAge"
                                     + "(p.firstName, p.age) FROM Person p";
        final TypedQuery<NameAndAge> query = entityManager.createQuery(createObjJPQL, NameAndAge.class);

        System.out.println("\nObj Create Query");
        final List<NameAndAge> result = query.getResultList();
        result.forEach(System.out::println);
    }

    private static void performNativeQuery(final EntityManager entityManager)
    {
        final String sqlString = "SELECT VORNAME, NAME FROM PersonenJPQL";
        final Query nativeQuery = entityManager.createNativeQuery(sqlString);

        System.out.println("\nNative Query");
        final List<Object[]> rows = nativeQuery.getResultList();
        rows.forEach(row -> System.out.println(Arrays.toString(row)));
    }

    private static void createPersons(final EntityManager entityManager)
    {
        final Person micha = new Person("Micha-JPQL", "Inden", new Date(71, 1, 7));
        final Person tim = new Person("Tim-JPQL", "Bötz", new Date(84, 2, 27));
        final Person tom = new Person("Tom-JPQL", "Meyer", new Date(55, 2, 31));

        entityManager.persist(micha);
        entityManager.persist(tim);
        entityManager.persist(tom);
    }
}
