package ch04.jpa.jpql_queries;

import java.util.Date;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.persistence.TypedQuery;

/**
 * Beispiel zur Demonstration von Paramtrierungen in JPQL-Queries
 * 
 * @author Michael Inden
 * 
 * Copyright 2012, 2016 by Michael Inden 
 */
public final class ParametrizedQueriesExample
{
    public static void main(final String[] args) throws Exception
    {
        final EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory("java-profi-PU-JPQL");
        final EntityManager entityManager = entityManagerFactory.createEntityManager();

        try
        {
            performJPAActions(entityManager);
        }
        finally
        {
            entityManager.close();
            entityManagerFactory.close();
        }
    }

    private static void performJPAActions(final EntityManager entityManager)
    {
        try
        {
            entityManager.getTransaction().begin();

            executeStatements(entityManager);

            entityManager.getTransaction().commit();
        }
        catch (Exception e)
        {
            entityManager.getTransaction().rollback();
        }
    }

    private static void executeStatements(final EntityManager entityManager)
    {
        createPersons(entityManager);

        final int minAge = 40;
        final int maxAge = 50;

        final String jpql1 = "SELECT person FROM Person person " + "WHERE person.age >= ? AND person.age < ?";
        final TypedQuery<Person> typedQuery1 = entityManager.createQuery(jpql1, Person.class);
        typedQuery1.setParameter(1, minAge);
        typedQuery1.setParameter(2, maxAge);
        typedQuery1.getResultList().forEach(System.out::println);

        final String jpql2 = "SELECT person FROM Person person "
                             + "WHERE person.age >= :minAge AND person.age < :maxAge";
        final TypedQuery<Person> typedQuery2 = entityManager.createQuery(jpql2, Person.class);
        typedQuery2.setParameter("minAge", minAge);
        typedQuery2.setParameter("maxAge", maxAge);
        typedQuery2.getResultList().forEach(System.out::println);
    }

    private static void createPersons(final EntityManager entityManager)
    {
        final Person micha = new Person("Micha-JPQL", "Inden", new Date(71, 1, 7));
        final Person tim = new Person("Tim-JPQL", "Bötz", new Date(84, 2, 27));
        final Person tom = new Person("Tom-JPQL", "Meyer", new Date(55, 2, 31));

        entityManager.persist(micha);
        entityManager.persist(tim);
        entityManager.persist(tom);
    }

    private static void performQueryAndPrintResults(final Query query)
    {
        final List<?> persons = query.getResultList();

        persons.forEach(System.out::println);
    }
}
