package ch04.jpa.jpql_queries;

import java.util.Date;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.persistence.TypedQuery;

/**
 * Beispiel zur Demonstration verschiederer Aktionen mit JPQL
 *
 * @author Michael Inden
 * 
 * Copyright 2012, 2016 by Michael Inden 
 */
public final class PersonJpqlQueriesUpdateAndDeleteExample
{
    public static void main(final String[] args) throws Exception
    {
        final EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory("java-profi-PU-JPQL");
        final EntityManager entityManager = entityManagerFactory.createEntityManager();

        try
        {
            performJPAActions(entityManager);
        }
        finally
        {
            entityManager.close();
            entityManagerFactory.close();
        }
    }

    private static void performJPAActions(final EntityManager entityManager)
    {
        try
        {
            entityManager.getTransaction().begin();

            executeStatements(entityManager);

            entityManager.getTransaction().commit();
        }
        catch (Exception e)
        {
            entityManager.getTransaction().rollback();
        }
    }

    private static void executeStatements(final EntityManager entityManager)
    {
        createPersons(entityManager);
        performQueries(entityManager);
        performUpdateAndDelete(entityManager);
    }

    private static void performQueries(final EntityManager entityManager)
    {
        // Einfache Abfrage
        final String personsQuery = "SELECT p FROM Person p";
        final TypedQuery<Person> persons = entityManager.createQuery(personsQuery, Person.class);
        final List<Person> allPersons = persons.getResultList();

        // Zugriffe auf einzelne Attribute
        final String firstNamesQuery = "SELECT p.firstName FROM Person p " + "ORDER BY p.firstName";
        final TypedQuery<String> firstNames = entityManager.createQuery(firstNamesQuery, String.class);
        final List<String> sortedFirstNames = firstNames.getResultList();

        // Berechnungen in Queries
        final String personCountQuery = "SELECT COUNT(p) FROM Person p";
        final Query countQuery = entityManager.createQuery(personCountQuery);
        final Object count = countQuery.getSingleResult();

        final String avgAgeQuery = "SELECT AVG(p.age) FROM Person p";
        final TypedQuery<Double> avgQuery = entityManager.createQuery(avgAgeQuery, Double.class);
        final Double avgAge = avgQuery.getSingleResult();

        // Ergebnisse ausgeben
        System.out.println(personsQuery + ":\n" + allPersons);
        System.out.println(firstNamesQuery + ":\n" + sortedFirstNames);
        System.out.println("\nCount: " + count + " / type: " + count.getClass());
        System.out.println("Avg Age: " + avgAge);
    }

    private static void performUpdateAndDelete(final EntityManager entityManager)
    {
        final String updateJPQL = "UPDATE Person p SET p.firstName = 'Mike' " + "WHERE p.firstName LIKE 'Micha%'";
        final Query updateCommand = entityManager.createQuery(updateJPQL);
        final int updatedRows = updateCommand.executeUpdate();

        final String deleteJPQL = "DELETE FROM Person p WHERE p.firstName LIKE 'T%'";
        final Query deleteCommand = entityManager.createQuery(deleteJPQL);
        final int deletedRows = deleteCommand.executeUpdate();

        System.out.println("\nUpdated rows: " + updatedRows);
        System.out.println("Deleted rows: " + deletedRows);
    }

    private static void createPersons(final EntityManager entityManager)
    {
        final Person micha = new Person("Micha-JPQL", "Inden", new Date(71, 1, 7));
        final Person tim = new Person("Tim-JPQL", "Bötz", new Date(84, 2, 27));
        final Person tom = new Person("Tom-JPQL", "Meyer", new Date(55, 2, 31));

        entityManager.persist(micha);
        entityManager.persist(tim);
        entityManager.persist(tom);
    }
}
