package ch04.jpa.jpql_queries;

import java.util.Date;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

/**
 * Beispiel zur Demonstration der Kurzvarainte beim Formulieren von Queries
 *  
 * @author Michael Inden
 * 
 * Copyright 2012, 2016 by Michael Inden
 */
public final class PersonShortQueryExample
{
    public static void main(final String[] args) throws Exception
    {
        final EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory("java-profi-PU-JPQL");
        final EntityManager entityManager = entityManagerFactory.createEntityManager();

        try
        {
            performJPAActions(entityManager);
        }
        finally
        {
            entityManager.close();
            entityManagerFactory.close();
        }
    }

    private static void performJPAActions(final EntityManager entityManager)
    {
        try
        {
            entityManager.getTransaction().begin();

            executeStatements(entityManager);

            entityManager.getTransaction().commit();
        }
        catch (Exception e)
        {
            entityManager.getTransaction().rollback();
        }
    }

    private static void executeStatements(final EntityManager entityManager)
    {
        createPersons(entityManager);

        final String shortQuery = "FROM Person";
        final List<Person> result1 = entityManager.createQuery(shortQuery, Person.class).getResultList();
        result1.forEach(System.out::println);

        final String normalQuery = "SELECT p FROM Person p";
        final List<Person> result2 = entityManager.createQuery(normalQuery, Person.class).getResultList();
        result2.forEach(System.out::println);
    }

    private static void createPersons(final EntityManager entityManager)
    {
        final Person micha = new Person("Micha-JPQL", "Inden", new Date(71, 1, 7));
        final Person tim = new Person("Tim-JPQL", "Inden", new Date(71, 2, 27));
        final Person werner = new Person("Werner-JPQL", "Inden", new Date(40, 0, 31));
        final Person barbara = new Person("Barabara-JPQL", "Inden", new Date(73, 2, 24));

        entityManager.persist(micha);
        entityManager.persist(werner);
        entityManager.persist(tim);
        entityManager.persist(barbara);
    }
}
