package ch04.jpa.orm.associations;

import java.util.Arrays;
import java.util.Date;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

/**
 * Beispiel zur Demonstration der Verarbeitung von Assoziationen mit JPA
 * 
 * @author Michael Inden
 * 
 * Copyright 2012, 2016 by Michael Inden 
 */
public final class AssociationsWithJpaExample
{
    public static void main(final String[] args) throws Exception
    {
        final EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory("java-profi-PU-Associations");
        final EntityManager entityManager = entityManagerFactory.createEntityManager();

        try
        {
        	performJPAActions(entityManager);
        }
        finally
        {
	        entityManager.close();
	        entityManagerFactory.close();
        }
    }

    private static void performJPAActions(final EntityManager entityManager) 
    {
    	try 
    	{
    		entityManager.getTransaction().begin();
    		
    	    executeStatements(entityManager);
    	    
    		entityManager.getTransaction().commit();
    	}
    	catch (Exception e) 
    	{
    		entityManager.getTransaction().rollback();
    	}
    }
    
private static void executeStatements(final EntityManager entityManager) 
{
	// Person-Objekte und Adressen-Objekte erzeugen
	final Person person1 = new Person("Mike", "Inden", new Date(71, 1, 7));
	final Person person2 = new Person("Tim", "Bötz", new Date(71, 2, 27));
	
	final Address address1 = new Address("Street 1", "Nr 1", 11111, "City 1");
	final Address address2 = new Address("Street 2", "Nr 2", 22222, "City 2");
	final Address address3 = new Address("Street 3", "Nr 3", 22222, "City 3");
	
	// Objekte miteinander verbinden => Assoziation 
	person1.setAddresses(Arrays.asList(address1, address2));
	person2.setAddresses(Arrays.asList(address3));
	
	// Objekte transitiv in Datenbank speichern
	entityManager.persist(person1);
	entityManager.persist(person2);
	
	// Personen aus Datenbank auslesen
	final List<Person> persons = 
			           entityManager.createQuery("FROM Person").getResultList();
	persons.forEach(System.out::println);
}
    
    private AssociationsWithJpaExample()
    {        
    }
}
