package ch05.mongo;

import java.util.function.Consumer;

import org.bson.Document;

import com.mongodb.MongoClient;
import com.mongodb.client.FindIterable;
import com.mongodb.client.MongoCollection;
import com.mongodb.client.MongoDatabase;

/**
 * Beispielklasse zum Einfügen und Lesen von Daten im JSON-Format mit MongoDB  
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden
 */
public class MongoInsertAndReadJSONExample
{
    public static void main(final String[] args)
    {
        try (final MongoClient client = new MongoClient("localhost", 27017))
        {
            final MongoDatabase database = client.getDatabase("mongoexample");
            final MongoCollection<Document> collection = database.getCollection("personsinsert");

            insertNameAsJSON(collection, "Tom");
            insertNameAsJSON(collection, "Tom");
            insertNameAsJSON(collection, "Michael");

            collection.insertOne(Document.parse("{'Invisible' : true}"));

            queryAllToms(collection);
        }
    }

    private static void insertNameAsJSON(final MongoCollection<Document> collection, final String name)
    {
        final String insertAsJsonString = "{ name: '" + name + "', 'friend': 'Jerry'}";
        final Document personDocument = Document.parse(insertAsJsonString);

        collection.insertOne(personDocument);
    }

    private static void queryAllToms(final MongoCollection<Document> collection)
    {
        final String queryAsJsonString = "{ name: 'Tom' }";
        final Document query = Document.parse(queryAsJsonString);

        final FindIterable<Document> results = collection.find(query);
        printAll(results);
    }

    private static void printAll(final FindIterable<Document> elements)
    {
        final Consumer<Document> consumer = System.out::println;
        elements.forEach(consumer);
    }
}