package ch05.mongo;

import java.io.IOException;

import org.bson.Document;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.mongodb.MongoClient;
import com.mongodb.client.MongoCollection;
import com.mongodb.client.MongoDatabase;

/**
 * Beispielklasse zum selbst realisierten ODM (Object Document Mapping) analog zum ORM
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden
 */
public class MongoPersonODMExample
{
    public static void main(final String[] args) throws IOException
    {
        try (final MongoClient client = new MongoClient("localhost", 27017))
        {
            final MongoDatabase db = client.getDatabase("mongoexample");
            final MongoCollection<Document> collection = db.getCollection("persons_odm");

            final Person micha = new Person("Michael", 44, "Bremen", "Zürich");
            final String json = storePersonInDb(collection, micha);
            System.out.println("To db: " + json);

            final Document query = Document.parse("{'age' : 44}");
            final Person readPerson = readPersonFromDb(collection, query);
            System.out.println("\nPerson from json/db: " + readPerson);
        }
    }

    /** realisiert die Umwandlung Person -> Json -> DB */
    private static String storePersonInDb(final MongoCollection<Document> collection, final Person person)
                    throws JsonProcessingException
    {
        final String jsonAsString = JsonMapper.toJsonString(person);
        collection.insertOne(Document.parse(jsonAsString));
        return jsonAsString;
    }

    /** realisiert die Umwandlung DB -> Json -> Person */
    private static Person readPersonFromDb(final MongoCollection<Document> collection, final Document query)
                    throws JsonParseException, JsonMappingException, IOException
    {
        final Document firstDoc = collection.find(query).first();
        System.out.println("\nDocument from db: " + firstDoc);

        if (firstDoc != null)
        {
            final String asJson = firstDoc.toJson();
            System.out.println("\nJson from db: " + asJson);

            return JsonMapper.fromJsonString(asJson, Person.class);
        }
        return null;
    }
}