package ch05.mongo.orm.spring;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;

/**
 * Beispielklasse zum ODM von Personen mit Spring Data MongoDB
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden
 */
@SpringBootApplication
public class ODMappingExample implements CommandLineRunner
{
    @Autowired
    private PersonRepository repository;

    public static void main(final String[] args)
    {
        SpringApplication.run(ODMappingExample.class, args);
    }

    @Override
    public void run(final String... args) throws Exception
    {
        create();
        read();
        update();
        delete();
    }

    protected void create()
    {
        repository.save(new Person("Michael", "Inden", 44));
        repository.save(new Person("Michael", "Meyer", 33));
        repository.save(new Person("Tim", "Meyer", 50));
        repository.save(new Person("Andi", "Meier", 35));
    }

    protected void read()
    {
        System.out.println("findAll():");
        System.out.println("----------");
        final List<Person> findAllResults = repository.findAll();
        printPersons(findAllResults);

        System.out.println("findByFirstname('Michael'):");
        System.out.println("---------------------------");
        final List<Person> findByFirstnameResults = repository.findByFirstname("Michael");
        printPersons(findByFirstnameResults);

        System.out.println("findByLastname('Meyer'):");
        System.out.println("------------------------");
        final List<Person> findByLastnameResults = repository.findByLastname("Meyer");
        printPersons(findByLastnameResults);

        System.out.println("findByAge(44):");
        System.out.println("--------------");
        final List<Person> findByAgeResults = repository.findByAge(44);
        printPersons(findByAgeResults);

        System.out.println("findByAgeGreaterThan(35):");
        System.out.println("-------------------------");
        final List<Person> findByAgeGTResults = repository.findByAgeGreaterThan(35);
        printPersons(findByAgeGTResults);

        System.out.println("findByAgeBetween(32, 36):");
        System.out.println("-------------------------");
        final List<Person> findByAgeRangeResults = repository.findByAgeBetween(32, 36);
        printPersons(findByAgeRangeResults);
    }

    protected void update()
    {
        System.out.println("update('Michael Inden'):");
        System.out.println("------------------------");
        final Person michaeli = repository.findByFirstnameAndLastname("Michael", "Inden");
        michaeli.changeFirstname("Micha");
        repository.save(michaeli);
        printPersons(repository.findAll());
    }

    protected void delete()
    {
        repository.deleteAll();
    }

    private void printPersons(final List<Person> persons)
    {
        persons.forEach(System.out::println);
        System.out.println();
    }
}