package ch05.mongo.orm.spring.step2;

import java.time.LocalDate;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;

/**
 * Beispielklasse zum ODM von Personen inkl. Adressen mit Spring Data MongoDB
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden
 */
@SpringBootApplication
public class PersonAndAddressesODMappingExample implements CommandLineRunner
{
    @Autowired
    private PersonAddressRepository repository;

    public static void main(final String[] args)
    {
       SpringApplication.run(PersonAndAddressesODMappingExample.class, args);
    }

	@Override
	public void run(final String... args) throws Exception
	{    
		createPersonsWithAddresses();
	    read();
	    
		repository.deleteAll();
	}
	
	protected void createPersonsWithAddresses() 
	{
		final Address aachen = new Address(State.DE, 52070, "Aachen");
		final Address zürich = new Address(State.CH, 8003, "Zürich");
		final Address kiel = new Address(State.DE, 24106, "Kiel");
		 
	    repository.save(new Person("Michael", "Inden", 
	    		                   LocalDate.of(1971, 2, 7), zürich, aachen));
	    repository.save(new Person("Michael", "Meyer", 
	    		                    LocalDate.of(1980, 6, 14), aachen));
	    repository.save(new Person("Tim", "Meyer", 
	    		                   LocalDate.of(1960, 3, 27), kiel));
	    repository.save(new Person("Andi", "Meier", 
	    		                   LocalDate.of(1984, 11, 26), kiel));
	}

protected void read() 
{
    System.out.println("findByAddresses('52070 Aachen'):");
    System.out.println("--------------------------------");
    final Address aachen = new Address(State.DE, 52070, "Aachen");
    printPersons(repository.findByAddresses(aachen));
    
    System.out.println("findByAddressesCity('Zürich'):");
    System.out.println("------------------------------");
    printPersons(repository.findByAddressesCity("Zürich"));
    
    System.out.println("findByAddressesZipCode(24106):");
    System.out.println("------------------------------");
    printPersons(repository.findByAddressesZipCode(24106));
    
    System.out.println("findByAddressesZipCodeAndOrderByFirstName(24106):");
    System.out.println("-------------------------------------------------");
    printPersons(repository.findByAddressesZipCodeOrderByFirstname(24106));
}
	
	private void printPersons(final List<Person> persons)
    {
		persons.forEach(System.out::println);
        System.out.println();
    }
}