package ch06.rest.jersey.client;

import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.Response;

/**
 * Beispielklasse eines einfachen REST-Clients (Zugriff auf einen REST-Server)
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden
 */
public class HelloClientExample
{
    public static void main(final String[] args) throws Exception
    {
        final String basePath = "http://localhost:7777/rest/";
        final String resourcePath = "greeting/Mike";

        final Client client = ClientBuilder.newClient();

        final String result = performGet(client, basePath, resourcePath);
        System.out.println("Response Content: " + result);
    }

    private static String performGet(final Client client, final String basePath, final String resourcePath)
    {
        final WebTarget webTarget = client.target(basePath).path(resourcePath);
        System.out.println("\nSending 'GET' request to URL '" + basePath + resourcePath + "'");

        final Response response = webTarget.request().get();
        final int responseCode = response.getStatus();
        System.out.println("Response Code: " + responseCode);

        if (responseCode != Response.Status.OK.getStatusCode())
        {
            throw new RuntimeException("HTTP error code: " + responseCode);
        }

        if (response.hasEntity())
        {
            return response.readEntity(String.class);
        }
        throw new IllegalStateException("response expected to contain data");
    }
}
