package ch06.rest.jersey.client;

import javax.ws.rs.client.Client;

import javax.ws.rs.client.ClientBuilder;

import javax.ws.rs.client.WebTarget;

import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

/**
 * Beispielklasse eines REST-Clients (Zugriff auf einen REST-Server)
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden
 */
public class HelloRESTClientWithFormats
{
    private static final String REST_URI                 = "http://localhost:7777/rest/";

    private static final String HELLO_MIKE_RESOUCRE_PATH = "greeting/Mike";

    public static void main(final String[] args)
    {
        final Client client = ClientBuilder.newClient();
        final WebTarget target = client.target(REST_URI);
        final WebTarget helloService = target.path(HELLO_MIKE_RESOUCRE_PATH);

        System.out.println("As Response:  " + getOutput(helloService));
        System.out.println("As XML:       " + getOutputAsXML(helloService));
        System.out.println("As JSON:      " + getOutputAsJSON(helloService));
        System.out.println("As Text:      " + getOutputAsText(helloService));
    }

    private static Response getOutput(final WebTarget service)
    {
        return service.request().accept(MediaType.APPLICATION_XML).get();
    }

    private static String getOutputAsXML(final WebTarget service)
    {
        return getOutputOfType(service, MediaType.APPLICATION_XML);
    }

    private static String getOutputAsJSON(final WebTarget service)
    {
        return getOutputOfType(service, MediaType.APPLICATION_JSON);
    }

    private static String getOutputOfType(final WebTarget service, final String type)
    {
        return service.request().accept(type).get(String.class);
    }

    private static String getOutputAsText(final WebTarget service)
    {
        return service.request().accept(MediaType.TEXT_PLAIN).get(String.class);
    }
}