package ch06.rest.jersey.server.advanced;

import java.lang.reflect.Method;
import java.util.logging.Logger;

import javax.ws.rs.container.ContainerRequestContext;
import javax.ws.rs.container.ContainerRequestFilter;
import javax.ws.rs.container.ResourceInfo;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.Status;
import javax.ws.rs.ext.Provider;

/**
 * Hilfsklasse zur Unterstützung bei Security
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden 
 */
@Provider
public class ServerAuthenticationRequestFilter implements ContainerRequestFilter 
{
    private static final Logger LOGGER = 
                    Logger.getLogger("ServerAuthenticationRequestFilter");

    @Context
    ResourceInfo resourceInfo;

    @Override
    public void filter(final ContainerRequestContext requestContext) 
    {
        final Method method = resourceInfo.getResourceMethod();
        if (method.isAnnotationPresent(ModificationAccess.class)) 
        {
            final String authorization = requestContext.getHeaderString("Authorization");
            
            // Prüfe auf Michael:Inden
            final String expectedAuthorization = "Basic TWljaGFlbDpJbmRlbg==";
			final boolean isValid = authorization != null && 
                                    authorization.equals(expectedAuthorization);
            
            if (!isValid)
            {
            	LOGGER.warning("Authentication FAILED!");
                
                final Response response = Response.status(Status.UNAUTHORIZED).build();
                
                requestContext.abortWith(response);      
            }
            else
            {
            	LOGGER.info("Authentication accepted!");
            }
        }
    }
}