package ch06.rest.jersey.server.basics;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

/**
 * Beispielklasse zur Definition einer REST-Ressource, die einen Gruß ausliefert
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden
 */
@Path("/greeting")
public class HelloWorldResource
{
    @GET
    @Path("/{name}")
    @Produces(MediaType.TEXT_PLAIN)
    public Response getGreetingAsText(@PathParam("name") final String name)
    {
        final String output = "Hello " + name;

        return Response.status(Response.Status.OK).entity(output).build();
    }

    @GET
    @Path("/{name}")
    @Produces(MediaType.APPLICATION_XML)
    public Response getGreetingAsXML(@PathParam("name") final String name)
    {
        final String output = "<greeting>\n" + 
                              "   <salutation>Hello</salutation>\n" + 
                              "   <name>" + name + "</name>\n" +
                              "</greeting>";

        return Response.status(Response.Status.OK).entity(output).build();
    }

    @GET
    @Path("/{name}")
    @Produces(MediaType.APPLICATION_JSON)
    public Response getGreetingAsJson(@PathParam("name") final String name)
    {
        final String output = "{ greeting : [{ \"salutation\" : \"Hello\" }," + 
                                            "{ \"name\" : \"" + name + "\"}] }";

        return Response.status(Response.Status.OK).entity(output).build();
    }   
}
