package ch07.exampleapp.highscores.iteration1;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.time.LocalDate;

/**
 * Beispielklasse zur Demonstration des Persistierens einzelner Highscores
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden 
 */
public class HighscorePersistExample
{
    public static final void main(final String[] args) throws IOException
    {
        final HighscoreManager manager = new HighscoreManager();

        manager.add(new Highscore("Michael", 7271, 10, LocalDate.of(2016, 2, 7)));
        manager.add(new Highscore("Werner", 1000, 8, LocalDate.of(2016, 1, 31)));

        performActionOnFile(manager, "resources/Highscores.ser");
    }

    private static void performActionOnFile(final HighscoreManager manager, final String fileName) throws IOException
    {
        try (final OutputStream os = new FileOutputStream(fileName);
                        final InputStream is = new FileInputStream(fileName))
        {
            System.out.println("After save to file: " + fileName);
            manager.saveTo(os);
            manager.getHighscores().forEach(System.out::println);

            System.out.println("\nAfter load from csv: ");
            manager.populateFromCsv("resources/Highscores.csv");
            manager.getHighscores().forEach(System.out::println);

            System.out.println("\nAfter load from file: " + fileName);
            manager.loadFrom(is);
            manager.getHighscores().forEach(System.out::println);
        }
    }
}