package ch07.exampleapp.highscores.iteration2;

import static ch07.exampleapp.highscores.iteration2.HighscoresFromCsvImporter.readHighscoresFromCsv;
import static java.util.Comparator.comparing;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

/**
 * Beispielklasse zur Demonstration der Verwaltung von Highscores-Daten
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden 
 */
public class HighscoreManager 
{
	final Comparator<Highscore> byPoints = comparing(Highscore::getPoints);
	final Comparator<Highscore> byDay = comparing(Highscore::getDay);
	final Comparator<Highscore> byLevel = comparing(Highscore::getLevel);
	final Comparator<Highscore> byPointsLevelAndDay = 
										byPoints.reversed().
			                            thenComparing(byLevel.reversed()).
			                            thenComparing(byDay);
			
	private final List<Highscore> highscores = new ArrayList<>();
	
	public List<Highscore> getHighscores()
	{
		// Kopie erzeugen: keine internen Daten herausgeben
		final List<Highscore> soredResults = new ArrayList<>(highscores);
		soredResults.sort(byPointsLevelAndDay);
		return soredResults;
	}
		
	public void add(final Highscore highscore)
	{
		highscores.add(highscore);
	}
		
	public void populateFromCsv(final String filePath)
    {      
        final List<Highscore> readHighscores = readHighscoresFromCsv(filePath);

        highscores.clear();
        highscores.addAll(readHighscores);
    }
	
	public void saveTo(final PersistMode persistMode, 
			           final OutputStream os) throws IOException
	{
		final HighscorePersister persister = new HighscorePersister();
		persister.persist(persistMode, os, highscores);
	}
	
	public void loadFrom(final PersistMode persistMode, 
			             final InputStream is) throws IOException
	{
		final HighscorePersister persister = new HighscorePersister();
		final List<Highscore> readHighscores = persister.readFrom(persistMode, is);
		
        highscores.clear();
        highscores.addAll(readHighscores);
	}
}