package ch07.exampleapp.highscores.iteration3;

import java.io.Serializable;
import java.time.LocalDate;
import java.util.concurrent.atomic.AtomicLong;

import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.adapters.XmlJavaTypeAdapter;

/**
 * Diese Klasse modelliert einen Highscore-Eintrag
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden
 */
@XmlRootElement
public final class Highscore implements Serializable
{
    private static final AtomicLong counter = new AtomicLong();
    
    private long              id;  
    
	private String name;
	private int points;
	private int level;
	private LocalDate day;

	// für XML-Serialisierung
    public Highscore()
    {
    }
    
	public Highscore(final String name, final int points, 
			         final int level, final LocalDate day) 
	{
		this.id = counter.getAndIncrement();
		this.name = name;
		this.points = points;
		this.level = level;
		this.day = day;
	}
  
    public long getId()
    {
        return id;
    }

    public void setId(long id)
    {
        this.id = id;
    }

    public String getName()
    {
        return name;
    }

    public void setName(String name)
    {
        this.name = name;
    }
    
    public int getPoints()
    {
        return points;
    }

    public void setPoints(int points)
    {
        this.points = points;
    }
    
    public int getLevel()
    {
        return level;
    }

    public void setLevel(int level)
    {
        this.level = level;
    }
	
	@XmlJavaTypeAdapter(LocalDateAdapter.class)  
    public LocalDate getDay()
    {
        return day;
    }

    public void setDay(final LocalDate day)
    {
        this.day = day;
    }    

	@Override
	public String toString() 
	{
		return "Highscore [name=" + name + ", points=" + points + ", level=" + level + ", date=" + day + "]";
	}
}