package ch07.exampleapp.highscores.iteration3;

import java.net.URI;
import java.time.LocalDate;
import java.util.List;
import java.util.Optional;

import javax.ws.rs.DELETE;
import javax.ws.rs.FormParam;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

/**
 * Diese Klasse definiert eine REST-Resource für Highscores
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden
 */
@Path("/highscores")
public class HighscoreRESTService 
{
	private final HighscoreManager highscoreManager = 
			                       ManagerProvider.getHighscoreManager();
	
    @POST
    public Response createHighscore(@FormParam("name") final String name, 
                                    @FormParam("points") final int points,
                                    @FormParam("level") final int level)
    {
        if (name.trim().length() > 0)
        {
            final Highscore highscore = new Highscore(name, points, level, 
            		                                  LocalDate.now());
            highscoreManager.add(highscore);

            return Response.created(URI.create("/rest/highscores/" + 
                                               highscore.getId())).build();
        }
        return Response.status(Response.Status.BAD_REQUEST).build();
    }
	
	@GET
	@Produces({ MediaType.APPLICATION_XML, MediaType.APPLICATION_JSON })
	public List<Highscore> getHighscores() 
	{
		return highscoreManager.getHighscores();
	}

	@GET
	@Path("/top10")
	@Produces({ MediaType.APPLICATION_JSON, MediaType.APPLICATION_XML })
	public List<Highscore> getTop10() 
	{
        return highscoreManager.getHighscoresTopN(10);
	}
    
    @DELETE
    @Path("/{id}")
    public Response deleteHighscoreById(@PathParam("id") final long id)
    {
        final Optional<Highscore> optHighscore = highscoreManager.findBy(id);
        if (optHighscore.isPresent())
        {
        	highscoreManager.delete(optHighscore.get());

            return Response.noContent().build();
        }

        return Response.status(Response.Status.NOT_FOUND).build();
    }
}
