package ch04.jpa.orm.validation;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.TypedQuery;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

/**
 * Beispiel der Validierung in Form eines Unit Tests 
 * 
 * @author Michael Inden
 * 
 * Copyright 2016 by Michael Inden 
 */
public class CustomerTest
{
    private static EntityManagerFactory emf;

    private EntityManager               em;

    @BeforeClass
    public static void createEntityManagerFactory()
    {
        emf = Persistence.createEntityManagerFactory("java-profi-PU-VALIDATION");
    }

    @AfterClass
    public static void closeEntityManagerFactory()
    {
        emf.close();
    }

    @Before
    public void beginTransaction()
    {
        em = emf.createEntityManager();
        em.getTransaction().begin();
    }

    @After
    public void rollbackTransaction()
    {
        em.getTransaction().rollback();
        em.close();
    }

    @Test
    public void validCustomer()
    {
        final Customer customer = new Customer("Michael");
        em.persist(customer);

        final TypedQuery<Customer> query = em.createNamedQuery("findAllCustomers", Customer.class);
        assertEquals(1, query.getResultList().size());
    }

    @Test
    public void nameTooShort()
    {
        try
        {
            final Customer customer = new Customer("One");
            em.persist(customer);

            fail("Expected ConstraintViolationException, but not thrown.");
        }
        catch (final ConstraintViolationException e)
        {
            final List<ConstraintViolation<?>> violations = getViolationsFrom(e);
            assertEquals(1, violations.size());

            final ConstraintViolation<?> firstViolation = violations.get(0);
            assertEquals("name", firstViolation.getPropertyPath().toString());
        }
    }

    @Test
    public void nameTooLong()
    {
        try
        {
            final Customer customer = new Customer("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
            em.persist(customer);

            fail("Expected ConstraintViolationException, but not thrown.");
        }
        catch (final ConstraintViolationException ex)
        {
            final List<ConstraintViolation<?>> violations = getViolationsFrom(ex);
            assertEquals(1, violations.size());

            final ConstraintViolation<?> firstViolation = violations.get(0);
            assertEquals("name", firstViolation.getPropertyPath().toString());
        }
    }

    private List<ConstraintViolation<?>> getViolationsFrom(final ConstraintViolationException ex)
    {
        return new ArrayList<>(ex.getConstraintViolations());
    }
}